@@ -521,7 +521,7 @@ class Executor(object):
         raise NotImplementedError()
 
     def map(self, fn, *iterables, timeout=None, chunksize=1):
-        """Returns a iterator equivalent to map(fn, iter).
+        """Returns an iterator equivalent to map(fn, iter).
 
         Args:
             fn: A callable that will take as many arguments as there are
